;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x6e556d32a47f16a85b371515b4afba47 . "./code.cg2js/cg-acache.fasl")
(#x2e4368d2cd5f66ac03ea49e7324e21de . "./code.cg2js/cg-base.fasl")
(#xfc1740a7284349d7b304e4910a3d540b . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x51f8f50608d167fd02a2a3f7e769cb27 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xb758ef417698e6ee05f5765460610d8c . "./code.cg2js/cg-bitmap-stream.fasl")
(#x8fc088877844f34c3bee964ee5764170 . "./code.cg2js/cg-button.fasl")
(#x3d07b6ae80dc287e75108b38849a40a8 . "./code.cg2js/cg-calendar.fasl")
(#x01eccc63a25c1aa138cadf4e8e61fb89 . "./code.cg2js/cg-caret.fasl")
(#xbc659b490b5b7cf5c3d0e218e5cc9ceb . "./code.cg2js/cg-chart-or-plot.fasl")
(#x305623bc93ee6add99ae0d17238c132c . "./code.cg2js/cg-chart-widget.fasl")
(#xdad34ae025e918c30f9b002fad0e6a02 . "./code.cg2js/cg-check-box.fasl")
(#x8b43e72cd45d2cdcb7f48744f3497bca . "./code.cg2js/cg-choice-list.fasl")
(#x55a60e8614f9d80a81c0ba5770580d1b . "./code.cg2js/cg-choose-printer.fasl")
(#x22f7c52e0cf24ab7734a5e68e0219e5e . "./code.cg2js/cg-class-grid.fasl")
(#x8a66a002e9e635869eae045fb2084cf3 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x8f25d84e68bd8134e60222d1034104d5 . "./code.cg2js/cg-class-support.fasl")
(#xa2a914d7d8adb78dce796ea408a97d4d . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xdbb02987e3e367538bb0f2fe618d228a . "./code.cg2js/cg-clipboard-stack.fasl")
(#xeb7af5023004530dd8c05590111600be . "./code.cg2js/cg-clipboard.fasl")
(#x8713bebe7c3e8a3802378748121b78b6 . "./code.cg2js/cg-color-dialog.fasl")
(#x44d9990e3e48acf2ad987b9cda33bdb2 . "./code.cg2js/cg-combo-box.fasl")
(#xaa1aa54cdd36821734ef7cd652b23ccd . "./code.cg2js/cg-common-control.fasl")
(#x758d67a95e5768424dc45d55751bf4ef . "./code.cg2js/cg-comtab.fasl")
(#x4f801510ee83662f2d5d673baf2c3a8b . "./code.cg2js/cg-cursor-pixmap.fasl")
(#xd05da5610d76510b8acbbb9107dd1694 . "./code.cg2js/cg-curve.fasl")
(#xb2bf45c99e95db5c702b7ec8bac05573 . "./code.cg2js/cg-dde-utils.fasl")
(#xb2029eed4aa36b64cf997b3823cb7436 . "./code.cg2js/cg-dev.fasl")
(#xd45f770b2284c8f7ab7a4e08a0f713e0 . "./code.cg2js/cg-dialog-item.fasl")
(#x21678fa8802a3fe3fbcbc78cd8e90215 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x22b19c8fa8ff2d745e674c725f9557ed . "./code.cg2js/cg-directory-dialog.fasl")
(#x114eea49b516047047ea98f03ef7a371 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xe2340d4b9c78faf6d765c177266facb0 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x5901422e3d0495290d1a9dfb6d96dc2f . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x295e83dfb5078b6ed808bbac67f937d2 . "./code.cg2js/cg-drawable.fasl")
(#xf990f9ea791fe1913d2c743eba71fc06 . "./code.cg2js/cg-dropping-outline.fasl")
(#x36bc78cdf46213e79590e6b883711abc . "./code.cg2js/cg-edit-in-place.fasl")
(#x852132d855437fa76da2830ac7bee3b2 . "./code.cg2js/cg-editable-text.fasl")
(#x209900d9120ad53f949d816e187d14a5 . "./code.cg2js/cg-file-dialog.fasl")
(#x20fbac6105f42f97cdb3cfc190444c37 . "./code.cg2js/cg-fill-texture.fasl")
(#xac3172ece384c430d56f39e82229489e . "./code.cg2js/cg-find-string-dialog.fasl")
(#x2193cd6f9fd4f5335c062d775cdb1f98 . "./code.cg2js/cg-font-dialog.fasl")
(#xb4a15a4fad0960154a3df42673139ba4 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x4eee81be6c1b3831096b6a837a29908d . "./code.cg2js/cg-get-pixmap.fasl")
(#xeed30ea3ecfc5738f73f71e851a5a3b9 . "./code.cg2js/cg-get-position.fasl")
(#x190cdcab37724e1a9b775e79c8bd0161 . "./code.cg2js/cg-graphics-context.fasl")
(#x2754dbddd212fdf4ddf4b9c9dae97337 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x74ae1bbbf43c428e2f3a2a0dcb27db07 . "./code.cg2js/cg-grid-widget.fasl")
(#xffe0a79d92278d28a4463b1dd9c57208 . "./code.cg2js/cg-group-box.fasl")
(#xf0d1b617430fd7276a0310ef19925cea . "./code.cg2js/cg-header-control.fasl")
(#x9e44fd2c0d8789c2031a806377968820 . "./code.cg2js/cg-hotspot.fasl")
(#xe257df30c7878b1cf56fa7d8366e4c4e . "./code.cg2js/cg-html-dialog.fasl")
(#x990ab346376df2bfea8d53f236669734 . "./code.cg2js/cg-html-widget.fasl")
(#xe70d491ee0f9994e19f4223b0d5a155a . "./code.cg2js/cg-icon-pixmap.fasl")
(#xff298d3919a4ca3dafa9fa3ba0ca5441 . "./code.cg2js/cg-icon.fasl")
(#x4ebe37c4534ff12746deba4ac9251714 . "./code.cg2js/cg-ie.fasl")
(#x5d2d247466acbd764b6acfebea777a14 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x2cf326425818a9d1dfd5c2087b70419e . "./code.cg2js/cg-intersect.fasl")
(#x2e56c793295de228c720d821e82390e5 . "./code.cg2js/cg-item-list.fasl")
(#x64f6e946180570561d40e5d905a037e7 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x78d8be81b12df579b5f77b3e0bae0efb . "./code.cg2js/cg-lamp.fasl")
(#x8e2e0014705a049ea082384b265008da . "./code.cg2js/cg-layout.fasl")
(#xdfc3003050af1fe7a5327a198ef5d238 . "./code.cg2js/cg-lettered-menu.fasl")
(#xa3c06459a120aef57b90061d946b7155 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x727930d863f846482a2a97403209a342 . "./code.cg2js/cg-lisp-text.fasl")
(#xb1937c60bc3a129cb5062ddc3e3afd74 . "./code.cg2js/cg-lisp-widget.fasl")
(#xd663005fdca83f4edfca0d260fa593af . "./code.cg2js/cg-list-view.fasl")
(#xff52f191316046bf34bc9adad9ede821 . "./code.cg2js/cg-mci.fasl")
(#x37c9275227ee949ac4168da2984e0bce . "./code.cg2js/cg-menu-tooltip.fasl")
(#x711a5a1eee9e85a0322e534cfb926070 . "./code.cg2js/cg-menu.fasl")
(#x688b051c3bb6e5ff776c406d9cf04086 . "./code.cg2js/cg-message-dialog.fasl")
(#x869c8059b674a1cecdf5d6201315138b . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x22efaab6f46576a50a5d25e4e9592f45 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x1ab3fcdacd3000753d09df4887390b79 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x51dc2ef55f7fdc2597b13d713547196e . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xc1d7c8c1de463c9c42af68e95a21a2a3 . "./code.cg2js/cg-multi-picture-button.fasl")
(#xe670c979f65e1b42124706159b2ecd0d . "./code.cg2js/cg-nodes.fasl")
(#xff16c7b99317873f595bf415f40620f1 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x431e1b46b7ccb625e916add2a9a36b3a . "./code.cg2js/cg-object-editor.fasl")
(#x166f6f53a5a4cedb083abba4f1d352a5 . "./code.cg2js/cg-ocx.fasl")
(#x7289588a258fbbf8d9c941fcac91c19d . "./code.cg2js/cg-os-widget.fasl")
(#xf8be8ad169ed640092ada0f434898750 . "./code.cg2js/cg-os-window.fasl")
(#x212d67ff4cd993d4acc6a22146861963 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#xd120522e6ce56c7f17ad3a91c8de387c . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xb6c2360304bc4d4bef99beced91a79f6 . "./code.cg2js/cg-outline.fasl")
(#xe271f8a4d8fc0dd5c4f8c3f076e53e27 . "./code.cg2js/cg-palette.fasl")
(#xf0adeb08c95b6dc3448a87740a7dd36b . "./code.cg2js/cg-paren-matching.fasl")
(#xefdcd6a746fccb20ccc314b96b7cfe76 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xc842c3503219d5504e653032bdca6530 . "./code.cg2js/cg-picture-widget.fasl")
(#x6dcf0981ddd66bb77a427ff70c34ef4f . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x2f7a1021564e3f7c968fae9ce704be09 . "./code.cg2js/cg-pixmap-printing.fasl")
(#xe5d7035edc19c55afb4d70d33ff7427b . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x84adf57527202614ceedcdb3a85d81e9 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xbde5111a3390c684b4dce7a4005d4413 . "./code.cg2js/cg-pixmap.fasl")
(#xa48361fe645d1273398a32292dff2ab2 . "./code.cg2js/cg-player.fasl")
(#x374c9ec30d1cc69dd4752b1fc605e9a8 . "./code.cg2js/cg-plot-widget.fasl")
(#xab2fcd8747c8f5c7128277c4b926033d . "./code.cg2js/cg-printing.fasl")
(#x52af9d5caf3b4421035187922af970d8 . "./code.cg2js/cg-progress-indicator.fasl")
(#xfe1fe6f1632a8bab6b8019106bea8fdb . "./code.cg2js/cg-project-window.fasl")
(#x11b29a972dc36383cca6f376684e2286 . "./code.cg2js/cg-property.fasl")
(#x985321becec1417df0e176396f4cd89b . "./code.cg2js/cg-radio-button.fasl")
(#x6d1a00195daf705791cbc32dddf8e93c . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x2bdeac21565fb50550d8fb06dafc68c9 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#xf3696207e6b01bd798e49be5a49c183a . "./code.cg2js/cg-rich-edit-pane.fasl")
(#xa992dbd007f0b5879fbd1c77b81151c8 . "./code.cg2js/cg-rich-edit.fasl")
(#xca5d624cee7d3c3a078a0e66e417c227 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xc2c76919b2d40e65e398d15917f59913 . "./code.cg2js/cg-scaling-stream.fasl")
(#x2d7eddccde5832aab1eb01a9ff6920df . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x48794bf8ef0c70cc85cbc2fcd5fbfb96 . "./code.cg2js/cg-scroll-bar.fasl")
(#x2eff33354890d7bac883919e79f21b90 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x0a637aeda7c9b791022933d8894d6cae . "./code.cg2js/cg-selected-object.fasl")
(#xeb785ce7b94d09952d69906c7532cf3d . "./code.cg2js/cg-shortcut-menu.fasl")
(#x76eae6d5667b7673f77e72ba58dd235b . "./code.cg2js/cg-split-bar.fasl")
(#xa28c5799c53a534045eaa7726eab640d . "./code.cg2js/cg-static-text.fasl")
(#x962d8ed3043071173e668d4382e3153f . "./code.cg2js/cg-status-bar.fasl")
(#x06377a02c00e8a5d745da08be082bb29 . "./code.cg2js/cg-string-dialog.fasl")
(#x1b90a350edd712d06775b3cb5e4a7263 . "./code.cg2js/cg-tab-control.fasl")
(#x693ce0d77197c78efb63b99fa4cb371c . "./code.cg2js/cg-template-string.fasl")
(#xa587fc129fb0fb33ea2b8881ca2afd3d . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x6e83db8d369ed10a168d90f49746ff9d . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xe4c8548fe0333bda4853431cf181bc07 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xeec4bdbea66f2081f4281f7d34bf6105 . "./code.cg2js/cg-text-or-combo.fasl")
(#x80326953c398e45d5cd7085f0a81ea8e . "./code.cg2js/cg-text-widget.fasl")
(#xe28a9f4335ae05c3deec994171552b9b . "./code.cg2js/cg-timer.fasl")
(#x7158e5c12715585e56896cb07b31482f . "./code.cg2js/cg-toggling-widget.fasl")
(#x032562eeed3c22df699de79bdf02a572 . "./code.cg2js/cg-toolbar.fasl")
(#x3166679c6570d91888e527ac56e6d58b . "./code.cg2js/cg-tooltip.fasl")
(#x5508c7586bb9136dce8ade53a0f86921 . "./code.cg2js/cg-touch.fasl")
(#xb03747e0c2aa3859a121c932353e9916 . "./code.cg2js/cg-trackbar.fasl")
(#x6b923a1ada65bf6832119beff5d19a63 . "./code.cg2js/cg-tray.fasl")
(#xc7c1790a4b68430cfdf832158299c489 . "./code.cg2js/cg-up-down-control.fasl")
(#x1a1cbe4f3166f5807bde3dc06ea21b53 . "./code.cg2js/cg-utility-dialog.fasl")
(#x4de9140f8def42c230e79e311a0269d8 . "./code.cg2js/cg-web-browser-dde.fasl")
(#x5d19a5a37b819fba592ca465169cf381 . "./code.cg2js/cg-web-browser.fasl")
(#x0581cb2b987ae46351f54a0b91df45c5 . "./code.cg2js/cg-wrap-string.fasl")
(#xc6d9827721e8bc1d5a36b8e4f9961a88 . "./code.cg2js/cg-yes-no-list.fasl")
(#xfe62f2345626bed2d85022eee79a4f80 . "./code.cg2js/cg.fasl")
(#x9fa3783f67a619fc384977cbf85f8fb3 . "./code.cg2js/cg-yes-no-string.fasl")
(#xc1f6728ab8285fd456d58adc0d8b8937 . "./code.idejs/ide-acache.fasl")
(#xda4c34d76289f14eae5faa7e6fddc74d . "./code.idejs/ide-apropos.fasl")
(#xc4c6840231ba2d329cd402e0d7a8274a . "./code.idejs/ide-base.fasl")
(#xaa16ad8ee9a778e9c3f249db1067645d . "./code.idejs/ide-class-browser.fasl")
(#xf94997114356605ae75e5474d5395185 . "./code.idejs/ide-class-grapher.fasl")
(#x20e67ba423882b99d7b712f184db63fb . "./code.idejs/ide-clipboard.fasl")
(#xc6caf578076c03fc00461192c3dc2270 . "./code.idejs/ide-dev.fasl")
(#x1daf8d2ac4cdeab57661e943a089f88e . "./code.idejs/ide-editor.fasl")
(#x962493a532c66dfaa0af71239c781248 . "./code.idejs/ide-find-definitions.fasl")
(#xf20829f274c06f9e343caa0d03ba35ca . "./code.idejs/ide-find-in-files.fasl")
(#x6a5fc3961798501f53b3bcdbaa744ec8 . "./code.idejs/ide-form.fasl")
(#x999056520cb6b78acf595622a615fd8c . "./code.idejs/ide-grapher.fasl")
(#x2059b8e6549d3f91872a37c3ae55e6b1 . "./code.idejs/ide-inspector.fasl")
(#x36f59d154b3437323e0847720fdaa802 . "./code.idejs/ide-intro.fasl")
(#x14f90641ca2ea29b04aec1a664f35e79 . "./code.idejs/ide-keyshort.fasl")
(#x46bd0e5bd15adb1361ed1f5726d2666b . "./code.idejs/ide-navigator.fasl")
(#x9eefcae7a26281ab00ac6ba5b852085d . "./code.idejs/ide-package-list.fasl")
(#x5a296ada6e17ea0ae3cad5dcba8bebe4 . "./code.idejs/ide-patch-dialog.fasl")
(#x78264d9fbcec6a592ee41e75aef2b7e3 . "./code.idejs/ide-process-browser.fasl")
(#x507d367c17abc59fb992f06289c493cc . "./code.idejs/ide-project.fasl")
(#xf0eabb4079f4eee2c87d7a3b3dfb15bc . "./code.idejs/ide-trace.fasl")
(#x010a3dab7c8efe70bea515494222eaad . "./code.idejs/ide-window-list.fasl")
(#xc69f395e86c2e0b8a245f1ea2a5c67cf . "./code.idejs/ide.fasl")
(#x3d70f7088b7ddd139f75c06fecd8ac58 . "./cgjs.html")
(#x55fea08d92e113f5a8381c271febbc9e . "./ssl-not-found.html")
(#x4e1b21622d7d9cc4e6fc09c7875f8430 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
